# NOTE: Before using the DC package, please install packages by the following R syntax:
# install.packages(c("data.table", "mgcv", "akima", "plyr", "mvtnorm", "Rcpp", "mvnfast"))
# download and install Rtools
# install DC package file: DC_1.0.zip


# load DC package
library("DC")

# EXAMPLE:

load("test_data.RData") # load "don't know" data

# The "RES" contains a 1000x40 matrix, where the first 20 columns contains the item response matrix and the last 20 columns containts the "don't konw" response matrix (1 or 0).
# The data are simulated by the GPCM and a bivariate normal distribution with correlation 0.7.

# create a design matrix to indicate which dimension is measured by each item:
BinaryMatrix = matrix(0,2,dim(RES)[2])
BinaryMatrix[1,1:20] = 1
BinaryMatrix[2,21:40] = 1

# specify the parameter of DC:
k = 4

# specify quadrature points:
quadrature_points = 61

# run EM algorithm for DC:
mod = DC(RES,model="gpcm"
           ,algorithm = "em"
           ,grid_range = c(-5,5)
           ,quadrature_points = quadrature_points
           ,difference = 1e-3
           ,type="irt",BinaryMatrix=BinaryMatrix
           ,verbose=1
           ,optimizer="nlminb"
           ,nonnormal=list(name="Davidian",K= k ,random= 10 ,algorithm="nlminb",algorithm_local="lbfgs")
)

# run EM algorithm for EH:
# replace "name="Davidian"" by "name="EH""

# run EM algorithm with normal distribution:
# comment out the line "nonnormal"


# Information criteria:
mod$AIC
mod$HQIC
mod$BIC

# item estimates of GPCM:
do.call(rbind,mod$x)[1:20,] # row is item, column1 is intercept, column2 is slope for dimension 1, column3 is slope for dimension 2.

# parameters of missingness model
do.call(rbind,mod$x)[21:40,] # row is item, column1 is intercept, column2 is slope for dimension 1, column3 is slope for dimension 2.

# EAP estimates for persons
fscore(mod) # row is person, column is dimension

###### density estimates in DC ######

# generate quadrature points first:
X = generate_grid(lower = -5, upper = 5, quadrature_points = quadrature_points, dimension = 2)

# density estimates (row is theta, column is gamma):
Density = matrix(Davidian(mod$Davidian_estimate,Qpatterns=X,Davidian_K=mod$Davidian_K),quadrature_points,quadrature_points)

# quadrature points for dimensions:
Theta = seq(-5, 5, length.out = quadrature_points)
Gamma = seq(-5, 5, length.out = quadrature_points)

# plot 3-D surf:
persp(x = Theta
      , y = Gamma
      , z = Density
      , xlim = c(-5,5)
      , ylim = c(-5,5)
      , theta = 40
      , ticktype = "detailed")


